#! /usr/bin/env python

import rosbag
import rospy
import ros_numpy
import sys
from sensor_msgs.msg import Image
from cv_bridge import CvBridge, CvBridgeError
import numpy as np
import numpy.matlib
import math
import copy
import cv2
from PIL import Image
from pyproj import Proj, transform

## global variables
#f = open(folder+'imglist.txt','w')
imgno = 0
imglist = []
cvbdg = CvBridge()
proj_UTMK = Proj(init='epsg:5178')
proj_WGS84 = Proj(init='epsg:4326')
initpt = np.array([988118.672713562, 1818806.4875518726]) ## origin of campus, just for convenience.

def write_gps(msg, f_gps):
    global proj_UTMK, proj_WGS84
    tnow = msg.header.stamp.to_sec()
    lat = msg.latitude
    lon = msg.longitude
    x, y = transform(proj_WGS84,proj_UTMK,lon,lat)
    f_gps.write('%.6f, %.6f, %.6f\n'%(x-initpt[0],y-initpt[1],tnow))

def write_images(msg, folder):
    global f, imgno, imglist, cvbdg
    imgno += 1
    if (imgno % 10) != 0:
        return
    tnow = msg.header.stamp.to_sec()
    cv_img = cvbdg.imgmsg_to_cv2(msg,desired_encoding="passthrough")
    cv_img = (cv_img/50).astype('uint8')
#    DIM = (1280, 1024)
#    K_1 = np.array([[702.6030497884977, 0., 644.9296487349911], [0., 703.4541726858521, 526.4572414665469], [0., 0., 1.]])
#    D_1 = np.array([[0.5424763304847061, -1.7515099175022195, 6.050489512760127,-5.786170959900578]])
    DIM = (640, 512)
    K_1 = np.array([[445.34173838383924, 0., 310.74708274781557], [0., 446.40695195454197, 249.54892754326676], [0., 0., 1.]])
    D_1 = np.array([[0.248994091117758, -2.909390816436668, 8.052903070857168, -7.363581411738435]])
    map1_1, map2_1 = cv2.fisheye.initUndistortRectifyMap(K_1, D_1, np.eye(3), K_1, DIM, cv2.CV_16SC2)
    undistorted_img = cv2.remap(cv_img, map1_1, map2_1, interpolation=cv2.INTER_LINEAR, borderMode=cv2.BORDER_CONSTANT)

    cv2.imwrite(folder+'%.6f.png'%(tnow), undistorted_img)
#    imglist.append([str(imgno) + "%06i.png"%(imgno)])
#    f.write("%12.12f, %06i\n"%(tnow,imgno))

def raw_to_kelvin(val):
    return ( 1428.0 / log( 408825.0 / ( val + 58.417 ) + 1.0 ) )

def main(seqname):
    bag = rosbag.Bag(seqname+'.bag')
    folder = 'img_' + seqname + '_ther/'
    f_gps = open(folder+'gpslist.txt','w')

    ## write images with timestamp assignment
#    for topic, msg, t in bag.read_messages(topics=['/gps']):
#        write_gps(msg, f_gps)

#    for topic, msg, t in bag.read_messages(topics=['/camera/image_color']):
    for topic, msg, t in bag.read_messages(topics=['/thermal/image_raw']):
        write_images(msg, folder)

    print('saved '+str(imgno)+' images timstamps to detections.')

    bag.close()

if __name__ == '__main__':
    main(sys.argv[1])
